/**
 * Eine allgemeine Liste.
 * 
 * @author Bernd Gramlich 
 * @version 2015-12-09
 */
public class Liste
{
    private Listenelement erstesListenelement;

    /**
     * Konstruktor.
     * Erzeuge eine leere Liste. Das ist also eine
     * Liste, die nur aus einem Abschluss besteht.
     */
    public Liste ()
    {
        erstesListenelement = new Abschluss ();
    }

    /**
     * Nimm den vordersten Knoten aus der Liste und rcke auf.
     * Gib den Knoteninhalt als Rckgabewert.
     * Falls die Liste leer ist, gib null zurck.
     */
    public Eintrag entnimmVorne ()
    {
        /*
         * Wir geben am Ende den ersten Eintrag in der Liste aus,
         * wenn er existiert.
         */
        Eintrag ausgabe;

        /*
         * Falls die Liste leer ist ...
         */
        if (erstesListenelement instanceof Abschluss)
        {
            /*
             * ... gib null aus ...
             */
            ausgabe = null;
        }
        /*
         * ... ansonsten ...
         */
        else
        {
            /*
             * ... gib den Inhalt des ersten Knotens aus.
             */
            ausgabe = erstesListenelement.gibInhalt ();
            ausgabe.werdeUnsichtbar ();

            /*
             * Der Nachfolger des ersten Knotens wird neuer erster
             * Knoten. 
             */
            erstesListenelement = erstesListenelement.gibNachfolger ();

            /*
             * Lasse die Liste nach links aufrcken.
             */
            verschiebeNachLinks ();
        }

        return ausgabe;
    }

    /**
     * Fge einen neuen Eintrag vorne an die Liste an.
     */
    public void fuegeVorneAn (Eintrag neuerEintrag)
    {
        verschiebeNachRechts ();
        
        erstesListenelement = new Knoten (erstesListenelement, neuerEintrag);
    }

    /**
     * Fge einen neuen Eintrag hinten an die Liste an.
     */
    public void fuegeHintenAn (Eintrag neuerEintrag)
    {
        erstesListenelement = erstesListenelement.fuegeHintenAn (neuerEintrag);
    }
    
    /**
     * Fge einen neuen Eintrag aufsteigend sortiert in die Liste ein.
     */
    public void fuegePassendEin (Eintrag neuerEintrag)
    {
       erstesListenelement = erstesListenelement.fuegePassendEin (neuerEintrag);
    }

    /**
     * Nenne die Lnge der Liste.
     */
    public int gibLaenge ()
    {
        return erstesListenelement.gibLaenge ();
    }
    
    /**
     * Verschiebe die Liste graphisch nach rechts.
     */
    private void verschiebeNachRechts ()
    {
        erstesListenelement.verschiebeNachRechts ();
    }

    /**
     * Verschiebe die Liste graphisch nach links.
     */
    private void verschiebeNachLinks ()
    {
        erstesListenelement.verschiebeNachLinks ();
    }
    
    /**
     * Drucke den Inhalt der Liste auf die Konsole.
     */
    public void druckeAufKonsole ()
    {
        erstesListenelement.druckeAufKonsole ();
    }
    
    /**
     * Zhle, wie viele Knoten die "eigenschaft" haben.
     * 
     * Beispiel:
     * zaehle (new IstWeiblich ())
     * zhlt die Anzahl der Frauen in der Liste.
     */
    public int zaehle (Eigenschaft eigenschaft)
    {
        return erstesListenelement.zaehle (eigenschaft);
    }
    
    /**
     * Fhre fr alle Knoten, die die Eigenschaft "eigenschaft"
     * haben, den Befehl "befehl" mit den Argumenten "zeichenkette"
     * aus.
     * 
     * Beispiele:
     * fuehreAusFuer (new IstWeiblich (), new Schwebe (), {});
     * lsst alle Frauen in der Liste nach oben schweben.
     * fuehreAusFuer (new IstWeiblich (), new FaerbeHaare (), {"rot"});
     * frbt die Haare aller Frauen in der Liste rot.
     * fuehreAusFuer (new IstPerson (), new SetzeKleiderfarben (),
     *                                    {"rot", "gelb", "grn"});
     * kleidet alle Personen in der Liste in Ampelfarben.
     */
    public void fuehreAusFuer (Eigenschaft eigenschaft, Befehl befehl, String[] zeichenkette)
    {
        erstesListenelement.fuehreAusFuer (eigenschaft, befehl, zeichenkette);
    }
}